import { useState, useRef, useEffect } from 'react';
import { isEmpty } from 'ramda';

function _extends() {
  _extends = Object.assign || function (target) {
    for (var i = 1; i < arguments.length; i++) {
      var source = arguments[i];

      for (var key in source) {
        if (Object.prototype.hasOwnProperty.call(source, key)) {
          target[key] = source[key];
        }
      }
    }

    return target;
  };

  return _extends.apply(this, arguments);
}

var generateField = function generateField(_ref) {
  var _fieldConfig$validati3, _field$isPristine, _field$hasError, _fieldConfig$isRequir, _fieldConfig$validate, _fieldConfig$initialV, _field$errorMessage;

  var fieldConfig = _ref.fieldConfig,
      field = _ref.field,
      setField = _ref.setField,
      _ref$parentKey = _ref.parentKey,
      parentKey = _ref$parentKey === void 0 ? '' : _ref$parentKey;

  var _computeErrorMessage4 = function computeErrorMessage(value, forceCheck) {
    var _firstError$errorMess;

    if (forceCheck === void 0) {
      forceCheck = false;
    }

    if (!forceCheck && field != null && field.isPristine && !fieldConfig.isRequired || !fieldConfig.validationRules) {
      return {
        errorMessage: '',
        hasError: false
      };
    }

    if (fieldConfig.isRequired && isEmpty(value)) {
      var _fieldConfig$validati, _fieldConfig$validati2;

      return {
        errorMessage: (_fieldConfig$validati = (_fieldConfig$validati2 = fieldConfig.validationRules[0]) == null ? void 0 : _fieldConfig$validati2.errorMessage) != null ? _fieldConfig$validati : '',
        hasError: true
      };
    }

    if (!fieldConfig.isRequired && !Boolean(value) && !fieldConfig.validationRules) {
      return {
        errorMessage: '',
        hasError: false
      };
    }

    var firstError = fieldConfig.validationRules.find(function (rule) {
      return !rule.validate(value);
    });
    return {
      errorMessage: (_firstError$errorMess = firstError == null ? void 0 : firstError.errorMessage) != null ? _firstError$errorMess : '',
      hasError: Boolean(firstError == null ? void 0 : firstError.errorMessage)
    };
  };

  return _extends({}, fieldConfig, {
    validationRules: (_fieldConfig$validati3 = fieldConfig.validationRules) != null ? _fieldConfig$validati3 : [],
    value: field ? field.value : fieldConfig.initialValue,
    hasChange: (field == null ? void 0 : field.value) !== (field == null ? void 0 : field.localInitialValue),
    isPristine: (_field$isPristine = field == null ? void 0 : field.isPristine) != null ? _field$isPristine : true,
    parentKey: parentKey,
    hasError: (_field$hasError = field == null ? void 0 : field.hasError) != null ? _field$hasError : false,
    isRequired: (_fieldConfig$isRequir = fieldConfig.isRequired) != null ? _fieldConfig$isRequir : false,
    validateOnBlur: (_fieldConfig$validate = fieldConfig.validateOnBlur) != null ? _fieldConfig$validate : false,
    localInitialValue: (_fieldConfig$initialV = fieldConfig.initialValue) != null ? _fieldConfig$initialV : '',
    errorMessage: (_field$errorMessage = field == null ? void 0 : field.errorMessage) != null ? _field$errorMessage : '',
    onChangeValue: function onChangeValue(value) {
      return setField(function (prevState) {
        var _fieldConfig$validate2;

        var newValue = fieldConfig.liveParser ? fieldConfig.liveParser(value) : value;
        return _extends({}, prevState, {
          value: newValue,
          hasChange: value !== (prevState == null ? void 0 : prevState.localInitialValue),
          isPristine: prevState != null && prevState.isPristine ? (_fieldConfig$validate2 = fieldConfig.validateOnBlur) != null ? _fieldConfig$validate2 : false : prevState == null ? void 0 : prevState.isPristine
        }, fieldConfig.validateOnBlur && !prevState.hasError ? {} : _computeErrorMessage4(newValue, true));
      });
    },
    onBlur: function onBlur() {
      if (!fieldConfig.validateOnBlur) {
        return;
      }

      var _computeErrorMessage = _computeErrorMessage4(field == null ? void 0 : field.value, true),
          hasError = _computeErrorMessage.hasError,
          errorMessage = _computeErrorMessage.errorMessage;

      var isFieldEmpty = !fieldConfig.isRequired && isEmpty(field == null ? void 0 : field.value);
      setField(function (prevState) {
        return _extends({}, prevState, {
          isPristine: isFieldEmpty,
          errorMessage: isFieldEmpty ? '' : errorMessage,
          hasError: isFieldEmpty ? false : hasError
        });
      });
    },
    validateOnSubmit: function validateOnSubmit() {
      var _computeErrorMessage2 = _computeErrorMessage4(field == null ? void 0 : field.value),
          hasError = _computeErrorMessage2.hasError,
          errorMessage = _computeErrorMessage2.errorMessage;

      setField(function (prevState) {
        return _extends({}, prevState, {
          errorMessage: errorMessage,
          hasError: hasError
        });
      });
      return {
        hasError: hasError,
        errorMessage: errorMessage
      };
    },
    onChangeInitialValue: function onChangeInitialValue(value) {
      if ((field == null ? void 0 : field.localInitialValue) === value) {
        return;
      }

      setField(function (prevState) {
        return _extends({}, prevState, {
          localInitialValue: value
        });
      });
    },
    setError: function setError(errorMessage) {
      return setField(function (prevState) {
        return _extends({}, prevState, {
          errorMessage: errorMessage,
          hasError: Boolean(errorMessage)
        });
      });
    },
    resetState: function resetState() {
      return setField(function (prevState) {
        return _extends({}, prevState, {
          isPristine: true,
          hasError: false,
          errorMessage: '',
          value: prevState.localInitialValue
        });
      });
    },
    validate: function validate() {
      var _computeErrorMessage3 = _computeErrorMessage4(field == null ? void 0 : field.value, true),
          hasError = _computeErrorMessage3.hasError,
          errorMessage = _computeErrorMessage3.errorMessage;

      setField(function (prevState) {
        return _extends({}, prevState, {
          errorMessage: errorMessage,
          hasError: hasError
        });
      });
      return hasError;
    },
    computeErrorMessage: function computeErrorMessage(value) {
      return _computeErrorMessage4(value, true);
    }
  });
};

function useField(fieldConfig) {
  var _useState = useState({
    value: fieldConfig.initialValue,
    localInitialValue: fieldConfig.initialValue,
    isPristine: true,
    errorMessage: '',
    hasError: false
  }),
      field = _useState[0],
      setField = _useState[1];

  return generateField({
    fieldConfig: fieldConfig,
    field: field,
    setField: setField
  });
}

/* eslint-disable  @typescript-eslint/no-explicit-any */
var isDefined = function isDefined(subject) {
  return typeof subject !== 'undefined' && subject !== null;
};

var ifDefined = function ifDefined(subject, then) {
  return isDefined(subject) && then(subject);
};

function useForm(formFields, callbacks) {
  var injectedForm = Object.entries(formFields).reduce(function (acc, _ref) {
    var _extends2;

    var key = _ref[0],
        value = _ref[1];
    return _extends({}, acc, (_extends2 = {}, _extends2[key] = value, _extends2));
  }, {});

  var _useState = useState({}),
      innerForm = _useState[0],
      setInnerForm = _useState[1];

  var innerFormRef = useRef(innerForm);

  var form = _extends({}, injectedForm, Object.entries(innerForm).reduce(function (acc, _ref2) {
    var _extends5;

    var key = _ref2[0],
        field = _ref2[1];
    var fieldKeys = field.parentKey.split('.').filter(Boolean);

    if (fieldKeys.length > 0) {
      var mappedField = fieldKeys.reduce(function (localAcc, localKey, index) {
        var _localAcc$localKey, _extends3;

        return _extends({}, acc, localAcc, (_extends3 = {}, _extends3[localKey] = _extends({}, acc[localKey], localAcc[localKey] || {}, index === fieldKeys.length - 1 ? {
          children: [].concat(((_localAcc$localKey = localAcc[localKey]) == null ? void 0 : _localAcc$localKey.children) || [], [field])
        } : {}), _extends3));
      }, {});
      var removeEmptyObjects = Object.keys(mappedField).filter(function (key) {
        return Object.keys(mappedField[key]).length;
      }).reduce(function (acc, key) {
        var _extends4;

        return _extends({}, acc, (_extends4 = {}, _extends4[key] = mappedField[key], _extends4));
      }, {});
      return _extends({}, acc, removeEmptyObjects);
    }

    return _extends({}, acc, (_extends5 = {}, _extends5[key] = field, _extends5));
  }, {}));

  var hasError = Object.values(form).filter(function (item) {
    return item.isRequired;
  }).some(function (item) {
    return Boolean(item.errorMessage);
  });

  var addFieldsRecurrence = function addFieldsRecurrence(field, parentKey) {
    var _ref3;

    if (field.children) {
      return field.children.reduce(function (acc, child) {
        return _extends({}, acc, addFieldsRecurrence(child, parentKey + "." + field.key));
      }, {});
    }

    return _ref3 = {}, _ref3[field.key] = generateField({
      fieldConfig: field,
      field: innerFormRef[field.key],
      parentKey: parentKey,
      setField: function setField(getNewState) {
        if (typeof getNewState !== 'function') {
          return;
        }

        setInnerForm(function (prevState) {
          var _extends6;

          return _extends({}, prevState, (_extends6 = {}, _extends6[field.key] = _extends({}, innerFormRef.current[field.key], getNewState(innerFormRef.current[field.key])), _extends6));
        });
      }
    }), _ref3;
  };

  useEffect(function () {
    innerFormRef.current = innerForm;
  }, [innerForm]);
  return {
    form: form,
    hasError: hasError,
    isFilled: Object.values(form).filter(function (item) {
      return item.isRequired;
    }).every(function (item) {
      return Boolean(item.value) && !isEmpty(item.value);
    }),
    isValid: Object.values(form).filter(function (item) {
      return item.isRequired;
    }).every(function (item) {
      return Boolean(item.value) && !isEmpty(item.value) && item.computeErrorMessage(item.value).hasError === false;
    }),
    formHasChanges: function formHasChanges() {
      return Object.values(form).some(function (field) {
        return field.hasChange;
      });
    },
    setError: function setError(field, errorMessage) {
      form[field].setError(errorMessage);
      ifDefined(callbacks.onError, function (onError) {
        var _onError;

        return onError((_onError = {}, _onError[field] = errorMessage, _onError));
      });
    },
    setFieldValue: function setFieldValue(field, value) {
      form[field].onChangeValue(value);
    },
    setFieldInitialValue: function setFieldInitialValue(field, value) {
      form[field].onChangeInitialValue(value);
    },
    addFields: function addFields(fields) {
      return setInnerForm(function (prevState) {
        return _extends({}, prevState, fields.reduce(function (acc, item) {
          return _extends({}, acc, addFieldsRecurrence(item, ''));
        }, {}));
      });
    },
    removeFieldIds: function removeFieldIds(fields) {
      return setInnerForm(function (prevState) {
        return Object.keys(prevState).reduce(function (acc, key) {
          var _extends7;

          if (fields.includes(key)) {
            return acc;
          }

          return _extends({}, acc, (_extends7 = {}, _extends7[key] = prevState[key], _extends7));
        }, {});
      });
    },
    resetForm: function resetForm() {
      return Object.keys(form).forEach(function (key) {
        return form[key].resetState();
      });
    },
    submit: function submit() {
      var errors = Object.keys(form).reduce(function (acc, fieldKey) {
        var _extends8;

        var field = form[fieldKey];

        var _field$validateOnSubm = field.validateOnSubmit(),
            hasError = _field$validateOnSubm.hasError,
            errorMessage = _field$validateOnSubm.errorMessage;

        if (!errorMessage) {
          return acc;
        }

        return _extends({}, acc, (_extends8 = {}, _extends8[fieldKey] = errorMessage || (hasError ? 'This field is required' : ''), _extends8));
      }, {});
      var hasErrors = Object.values(errors).length > 0;

      if (hasErrors) {
        return ifDefined(callbacks.onError, function (onError) {
          return onError(errors);
        });
      }

      var parsedForm = Object.keys(form).reduce(function (acc, fieldKey) {
        var _extends9;

        var _form$fieldKey = form[fieldKey],
            value = _form$fieldKey.value,
            submitParser = _form$fieldKey.submitParser;
        return _extends({}, acc, (_extends9 = {}, _extends9[fieldKey] = submitParser ? submitParser(value) : value, _extends9));
      }, {});
      callbacks.onSuccess(parsedForm);
    },
    validateAll: function validateAll() {
      return Object.values(form).some(function (field) {
        return field.validate();
      });
    }
  };
}

export { useField, useForm };
//# sourceMappingURL=react-form-builder-v2.esm.js.map
